/*
 * LASER HARP - PM 2017
 * Calota Florin, 332CB
 */

#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>

#define NUM_NOTE		8
#define DURATA_NOTE		1

#define AREF 5
#define THRESHOLD 1

int frecvente_note[NUM_NOTE] = {
	523, 587, 659, 698, 783, 880, 987, 1046
};

/* Genereaza o nota folosind buzzer-ul. */
void buzzer_reda(int nota) {

	int perioada_nota = 1000000/frecvente_note[nota];
	int j;

	PORTB |= (1 << PB7);
	for (j = 0; j < perioada_nota / 2; ++j)
		_delay_us(1);

	PORTB &= ~(1 << PB7);
	for (j = 0 ; j < perioada_nota / 2; ++j)
		_delay_us(1);

}

int ADC_get(uint8_t channel)
{
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC));

	return ADC;
}

double ADC_voltage(int raw)
{
	return raw * AREF / 1024.;
}

void IO_init()
{
	/* BUZZER */
	{
		/* Setez pin-ul PB7 ca iesire */
		DDRB |= (1 << PB7);

		/* Opresc buzzer-ul */
		PORTB &= ~(1 << PB7);
	}

	/* FOTOREZISTENTE */
	{
		/*Setez pinii PA0-PA7 ca intrare*/
		DDRA &= ~(1 << PA0);
		DDRA &= ~(1 << PA1);
		DDRA &= ~(1 << PA2);
		DDRA &= ~(1 << PA3);
		DDRA &= ~(1 << PA4);
		DDRA &= ~(1 << PA5);
		DDRA &= ~(1 << PA6);
		DDRA &= ~(1 << PA7);

		/* Activez rezistenta de pull-up pentru pinii PA0-PA7*/
		PORTA |= (1 << PA0);
		PORTA |= (1 << PA1);
		PORTA |= (1 << PA2);
		PORTA |= (1 << PA3);
		PORTA |= (1 << PA4);
		PORTA |= (1 << PA5);
		PORTA |= (1 << PA6);
		PORTA |= (1 << PA7);
	}
}

void ADC_init(void)
{
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

void play_harp()
{
	while (1) {
		int i;
		double voltage_value;
		for (i = 0; i < NUM_NOTE; ++i) {
			/* Citeste tensiunea de pe canal */
			voltage_value = ADC_voltage(ADC_get(i));
			
			/* Daca tensiunea se afla deasupra pragului, reda nota */
			if (voltage_value > THRESHOLD)
				buzzer_reda(i);
		}
	}
}

int main(void) {

	IO_init();
	ADC_init();
	play_harp();

	return 0;
}
